import { Subscribable, Observable } from 'rxjs';
import { PartialModel, TextModel } from './model-service';
import { Selection } from '../plainTypes';
export type Viewer = CodeEditor | DirectoryViewer;
export type ViewerData = CodeEditorData | DirectoryViewerData;
/**
 * ViewerId exposes the unique ID of a viewer.
 */
export interface ViewerId {
    /** The unique ID of the viewer. */
    readonly viewerId: string;
}
export interface BaseViewerData {
    readonly isActive: boolean;
}
export interface DirectoryViewerData extends BaseViewerData {
    readonly type: 'DirectoryViewer';
    /** The URI of the directory that this viewer is displaying. */
    readonly resource: string;
}
/**
 * Describes a code viewer to be created.
 */
export interface CodeEditorData extends BaseViewerData {
    readonly type: 'CodeEditor';
    /** The URI of the model that this viewer is displaying. */
    readonly resource: string;
    readonly selections: Selection[];
}
/**
 * Describes a code editor that has been added to the {@link ViewerServce}.
 */
export interface CodeEditor extends ViewerId, CodeEditorData {
}
/**
 * Describes a directory viewer that has been added to the {@link IViewerService}.
 */
export interface DirectoryViewer extends ViewerId, DirectoryViewerData {
}
export type ViewerWithPartialModel = CodeEditorWithPartialModel | DirectoryViewer;
/**
 * A code editor witha a partial model.
 *
 * To get the editor's full model, look up the model in the {@link ModelService}.
 */
export interface CodeEditorWithPartialModel extends CodeEditor {
    model: PartialModel;
}
/**
 * A code editor with its full model, including the model text.
 */
export interface CodeEditorWithModel extends CodeEditor {
    /** The code editor's model. */
    model: TextModel;
}
export type ViewerUpdate = ({
    type: 'added';
    viewerData: ViewerData;
} & ViewerId) | ({
    type: 'updated';
    viewerData: Pick<CodeEditorData, 'selections'>;
} & ViewerId) | ({
    type: 'deleted';
} & ViewerId);
/**
 * The viewer service manages viewers.
 */
export interface IViewerService {
    /**
     * A map of all known viewers, indexed by viewerId.
     *
     * This is mostly used for testing, most consumers should use
     * {@link IViewerService#viewerUpdates} or {@link IViewerService#activeViewerUpdates}
     */
    readonly viewers: ReadonlyMap<string, Viewer>;
    /**
     * An observable of all viewer updates.
     *
     * Emits when a viewer is added, updated or removed.
     */
    readonly viewerUpdates: Subscribable<ViewerUpdate[]>;
    /**
     * An observable of updates to the active viewer.
     *
     * Emits the active viewer if there is one or `undefined` otherwise.
     */
    readonly activeViewerUpdates: Subscribable<Viewer | undefined>;
    /**
     * Add a viewer.
     *
     * @param viewer The description of the viewer to add.
     * @returns The added code viewer (which must be passed as the first argument to other
     * {@link ViwerService} methods to operate on this viewer).
     */
    addViewer(viewer: ViewerData): ViewerId;
    /**
     * Observe a viewer for changes.
     *
     * @param viewer The viewer to observe.
     * @returns An observable that emits when the viewer changes
     * and complets when the viewer is removed.
     * If no such viewer exists, it emits an error.
     */
    observeViewer(viewer: ViewerId): Observable<ViewerData>;
    /**
     * Sets the selections for a CodeEditor.
     *
     * @param codeEditor The editor for which to set the selections.
     * @param selections The new selections to apply.
     * @throws If no editor exists with the given editor ID.
     * @throws If the editor ID is not a CodeEditor.
     */
    setSelections(codeEditor: ViewerId, selections: Selection[]): void;
    /**
     * Remove a viewer.
     * Also removes the corresponding model if no other viewer is referencing it.
     *
     * @param viewer The viewer to remove.
     */
    removeViewer(viewer: ViewerId): void;
    /**
     * Remove all viewers.
     */
    removeAllViewers(): void;
}
